# Modern Applied Biostatistical Methods Using S-PLUS
# S. Selvin, Oxford, 1998
#
# Chapter 4: General Linear Models
#
# Time-stamp: <2008-03-24 13:46:11 chl>

###
### not in the book
a <- read.table("dbp.txt",header=T)
dbp <- a$dbp
wt <- a$weight
###

plot(wt,dbp,pch="*",xlab="weight",ylab="blood pressure",
     main="Five S-function summaries of a bivariate plot of points")
abline(lsfit(wt,dbp))

round(lsfit(wt,dbp)$coef,3)
round(cor(wt,dbp),3)

cor.test(wt,dbp)

                                        # Fig. 4.1
###
### Note:
### I adapt somewhat the code to draw the figure in one step.
### Note also that identify() cannot be used with png or postscript device,
### hence the little tricks with text().
###
plot(wt,dbp,pch="*",xlab="weight",ylab="blood pressure",
     main="Five S-function summaries of a bivariate plot of points")
abline(lsfit(wt,dbp))
identify(wt,dbp,labels=1:length(wt))
obs <- c(5,11,16,17,20)
dx <- c(1.5,-8,1.5,1.5,-8)
dy <- c(0,0,1,-1,0)
text(wt[obs]+dx,dbp[obs]+dy,obs,pos=4)

round(lsfit(wt,dbp)$coef,3)
round(lsfit(wt[-11],dbp[-11])$coef,3)

b <- lsfit(cbind(wt,wt^2),dbp)$coef
x <- seq(min(wt),max(wt),10)
lines(x,b[1]+b[2]*x+b[3]*x^2,lty=2)

x <- tapply(wt,cut(wt,3),median)
y <- tapply(dbp,cut(wt,3),median)
lines(x,y,type="b",pch="o",lty=3)

library(robustbase)
b <- ltsReg(wt,dbp,intercept=T)$coef
lines(x,b[1]+b[2]*x,lty=4)

lines(supsmu(wt,dbp),lty=5)

legend(270,85,c("least squares","quadratic","median","trimmed","supsmu"),
       lty=1:5)

dframe <- read.table("bwt.data",header=T)
attach(dframe)
otemp <- glm(bwt~ht+wt+gain,family=gaussian)
otemp

summary(otemp,correlation=F)

round(cor(dframe),3)

otemp0 <- lm(bwt~ht+wt+gain)
summary(otemp0)

for (j in 1:ncol(x)) {
  y <- y[!is.na(x[,j])]
  x <- x[!is.na(x[,j])]
}

rdev <- otemp$deviance
rdev
df0 <- otemp$df.residual
df0

R <- cor(bwt,predict(otemp))
R
R^2

1-otemp$deviance/otemp$null.deviance

                                        # Fig. 4.2
plot(bwt,pedict(otemp),pch="x",xlab="birth weight",
     ylab="predicted birth weights",
     main="Additive model -- predicted plotted against observed values")
text(1.5,3.8,paste("multiple correlation coefficient =",round(R,3)))

b <- otemp$coefficients
sd <- sqrt(diag(var(dframe)))
bstd <- b[-1]*sd[-1]
bstd

estimates <- otemp$coefficients
df <- otemp$df.residual
tvalues <- summary(otemp)$coefficients[,3]
pvalues <- 2*(1-pt(abs(tvalues),df))
round(cbind(estimates,tvalues,pvalues),4)

ss1 <- otemp$deviance
ss1
df1 <- otemp$df.residual
otemp0 <- glm(bwt~gain,family=gaussian)
ss0 <- otemp0$deviance
ss0
df0 <- otemp0$df.residual
q <- df0-df1
Fstat <- ((ss0-ss1)/q)/(ss1/df1)
pvalue <- 1-pf(Fstat,q,df1)
round(cbind(Fstat,q,df1,pvalue),3)

                                        # Fig. 4.3
se <- sqrt(otemp$deviance/otemp$df.residual)
rvalues <- otemp$residuals/se
plot(rvalues,ylim=c(-3,3),ylab="residual values",xlab="index",
     main="Plot of consecutive residuals")
abline(h=0)

n <- length(otemp$residuals)
test <- sum(ifelse(otemp$residuals>0,1,0))
binom.test(test,n)

                                        # Fig. 4.4
qqnorm(sort(rvalues),type="l",main="qqnorm (additive model)")
n <- 232
for (i in 1:20) {
  par(new=T)
  qqnorm(rnorm(n),pch=".",ylab="",yaxt="n")
}

otemp1 <- glm(bwt~wt+wt^2+ht+ht^2+gain+gain^2,family=gaussian)
summary(otemp1,correlation=F)

otemp <- glm(y~x,family=gaussian)

h <- lm.influence(lm(y~x))$hat
h <- ls.diag(lsfit(y~x))$hat

h <- lm.influence(lm(bwt~cbind(wt,ht,gain)))$hat
hord <- order(h)
n <- length(h)
m <- hord[n:(n-4)]
m
cbind(round(h[m],3),bwt[m],wt[m],ht[m],gain[m])

se <- NULL
for (i in 1:nrow(x)) {
  otemp <- glm(y[-i]~x[-i,],family=gaussian)
  se[i] <- sqrt(otemp$deviance/otemp$df.residual)
}

res <- otemp$residuals
resnorm <- res/s
resstd <- res/(s*sqrt(1-h))
se0 <- sqrt((n-k)*s^2/(n-k-1)-res^2/((n-k-1)*(1-h)))
resstud <- res/(se0*sqrt(1-h))

round(cbind(bwt,predict(otemp),res,resnorm,resstd,resstud),3)

d.gain <- data.frame(ht=c(163,163,163),wt=c(58,58,58),gain=c(12,15,18))
yhat <- predict(otemp,d.gain)
yhat

se <- predict(otemp,d.gain,se=T)$se.fit
se

df <- otemp$df.residual
yhat-qt(0.975,df)*se
yhat+qt(0.975,df)*se

###
### - R --

###
pointwise(predict(otemp,d.gain,se=T),coverage=0.95)$lower
pointwise(predict(otemp,d.gain,se=T),coverage=0.95)$upper

###
### not in the book
a <- read.csv("perinatal.csv",header=T,sep=",")
p <- c(a$prob[a$infants=="black"],a$prob[a$infants=="white"])
###

x0 <- c(800,seq(850,4250,100),4400)
x <- c(x0,x0)
n <- length(x0)
g <- c(rep(1,n),rep(0,n))
y <- log(-log(p))

###
### - R --
x0 <- a$weight
x <- c(x0,x0)
g <- c(rep(1,n),rep(0,n))
otemp <- glm(y~g+poly(x,4))
###

summary(otemp,correlation=F)

cor(y,predict(otemp))

b <- otemp$coefficients
